@@ -1,6 +1,6 @@
 package Net::DNS::Resolver::Base;
 #
-# $Id: Base.pm,v 1.4 2003/08/10 15:20:24 ctriv Exp $
+# $Id: Base.pm,v 1.10 2003/12/11 08:43:18 ctriv Exp $
 #
 
 use strict;
@@ -19,7 +19,7 @@ use Net::DNS;
 use Net::DNS::Packet;
 use Net::DNS::Select;
 
-$VERSION = (qw$Revision: 1.4 $)[1];
+$VERSION = (qw$Revision: 1.10 $)[1];
 
 #
 # Set up a closure to be our class data.
@@ -52,6 +52,7 @@ $VERSION = (qw$Revision: 1.4 $)[1];
 		axfr_rr        => [],
 		axfr_soa_count => 0,
 		persistent_tcp => 0,
+		persistent_udp => 0,
 		dnssec         => 0,
 		udppacketsize  => 0,  # The actual default is lower bound by Net::DNS::PACKETSZ
 		cdflag         => 1,  # this is only used when {dnssec} == 1
@@ -88,6 +89,7 @@ my %public_attr = map { $_ => 1 } qw(
 	tcp_timeout
 	udp_timeout
 	persistent_tcp
+	persistent_udp
 	dnssec
 );
 
@@ -555,26 +557,33 @@ sub send_udp {
 	my $srcport = $self->{'srcport'};
 	my $srcaddr = $self->{'srcaddr'};
 
-	# IO::Socket carps on errors if Perl's -w flag is turned on.
-	# Uncomment the next two lines and the line following the "new"
-	# call to turn off these messages.
+	my $sock;
 
-	#my $old_wflag = $^W;
-	#$^W = 0;
+	if ($self->persistent_udp && $self->{'sockets'}{'UDP'}) {
+		$sock = $self->{'sockets'}{'UDP'};
+		print ";; using persistent socket\n"
+			if $self->{'debug'};
+	} else {
+		# IO::Socket carps on errors if Perl's -w flag is turned on.
+		# Uncomment the next two lines and the line following the "new"
+		# call to turn off these messages.
 
-	# XXX Why is PeerPort defined here?
-	my $sock = IO::Socket::INET->new(
-			    PeerPort  => $dstport,
-			    LocalAddr => $srcaddr,
-			    LocalPort => ($srcport || undef),
-			    Proto     => 'udp',
-	);
+		#my $old_wflag = $^W;
+		#$^W = 0;
 
-	#$^W = $old_wflag;
+		$sock = IO::Socket::INET->new(
+				    LocalAddr => $srcaddr,
+				    LocalPort => ($srcport || undef),
+				    Proto     => 'udp',
+		);
 
-	unless ($sock) {
-		$self->errorstring("couldn't create socket: $!");
-		return;
+		#$^W = $old_wflag;
+
+		unless ($sock) {
+			$self->errorstring("couldn't create socket: $!");
+			return;
+		}
+		$self->{'sockets'}{'UDP'} = $sock if ($self->persistent_udp);
 	}
 
 	my @ns = grep { $_->[0] && $_->[1] }
@@ -827,155 +836,16 @@ sub axfr {
 		while (($rr, $err) = $self->axfr_next, $rr && !$err) {
 			push @zone, $rr;
 		}
-		@zone = () if $err && $err ne 'no zone transfer in progress';
+		@zone = () if $err;
 	}
 
 	return @zone;
 }
 
 sub axfr_old {
-	warn "Use of " . __PACKAGE__ . "::axfr_old() is deprecated.  Use axfr() or axfr_start().\n";
-	
-	my $self = shift;
-	my ($dname, $class) = @_;
-	$dname ||= $self->{'searchlist'}->[0];
-	$class ||= 'IN';
-
-	unless ($dname) {
-		print ";; ERROR: axfr: no zone specified\n" if $self->{'debug'};
-		$self->errorstring('no zone');
-		return;
-	}
-
-	print ";; axfr($dname, $class)\n" if $self->{'debug'};
-
-	unless (@{$self->{'nameservers'}}) {
-		$self->errorstring('no nameservers');
-		print ";; ERROR: no nameservers\n" if $self->{'debug'};
-		return;
-	}
-
-	my $packet = $self->make_query_packet($dname, 'AXFR', $class);
-	my $packet_data = $packet->data;
-
-	my $ns = $self->{'nameservers'}->[0];
-
-	print ";; axfr nameserver = $ns\n" if $self->{'debug'};
-
-	my $srcport = $self->{'srcport'};
-
-	my $sock;
-	my $sock_key = "$ns:$self->{'port'}";
-
-	if ($self->{'persistent_tcp'} && $self->{'sockets'}{$sock_key}) {
-		$sock = $self->{'sockets'}{$sock_key};
-		print ";; using persistent socket\n" if $self->{'debug'};
-	}
-	else {
-
-		# IO::Socket carps on errors if Perl's -w flag is turned on.
-		# Uncomment the next two lines and the line following the "new"
-		# call to turn off these messages.
-
-		my $old_wflag = $^W;
-		$^W = 0;
-
-		$sock = IO::Socket::INET->new(
-		    PeerAddr  => $ns,
-		    PeerPort  => $self->{'port'},
-		    LocalAddr => $self->{'srcaddr'},
-		    LocalPort => ($srcport || undef),
-		    Proto     => 'tcp',
-		    Timeout   => $self->{'tcp_timeout'}
-		);
-
-		$^W = $old_wflag;
-
-		unless ($sock) {
-			$self->errorstring(q|couldn't connect|);
-			return;
-		}
-
-		$self->{'sockets'}{$sock_key} = $sock;
-	}
-
-	my $lenmsg = pack('n', length($packet_data));
-
-	unless ($sock->send($lenmsg)) {
-		$self->errorstring($!);
-		return;
-	}
-
-	unless ($sock->send($packet_data)) {
-		$self->errorstring($!);
-		return;
-	}
-
-	my $sel = Net::DNS::Select->new($sock);
-
-	my @zone;
-	my $soa_count = 0;
-	my $timeout = $self->{'tcp_timeout'};
-
-	while (1) {
-		my @ready = $sel->can_read($timeout);
-		unless (@ready) {
-			$self->errorstring('timeout');
-			return;
-		}
-
-		my $buf = read_tcp($sock, &Net::DNS::INT16SZ, $self->{'debug'});
-		last unless length($buf);
-		my ($len) = unpack('n', $buf);
-		last unless $len;
-
-		@ready = $sel->can_read($timeout);
-		unless (@ready) {
-			$self->errorstring('timeout');
-			return;
-		}
-
-		$buf = read_tcp($sock, $len, $self->{'debug'});
-
-		print ';; received ', length($buf), " bytes\n"
-			if $self->{'debug'};
-
-		unless (length($buf) == $len) {
-			$self->errorstring("expected $len bytes, received " . length($buf));
-			return;
-		}
-
-		my ($ans, $err) = Net::DNS::Packet->new(\$buf, $self->{'debug'});
-
-		if (defined $ans) {
-			if ($ans->header->ancount < 1) {
-				$self->errorstring($ans->header->rcode);
-				last;
-			}
-		}
-		elsif (defined $err) {
-			$self->errorstring($err);
-			last;
-		}
-
-		foreach ($ans->answer) {
-			# $_->print if $self->{'debug'};
-			if ($_->type eq 'SOA') {
-				++$soa_count;
-				push @zone, $_ unless $soa_count >= 2;
-			}
-			else {
-				push @zone, $_;
-			}
-		}
-
-		last if $soa_count >= 2;
-	}
-
-	return @zone;
+	croak "Use of Net::DNS::Resolver::axfr_old() is deprecated, use axfr() or axfr_start().";
 }
 
-
 sub axfr_start {
 	my $self = shift;
 	my ($dname, $class) = @_;
@@ -1065,11 +935,14 @@ sub axfr_start {
 sub axfr_next {
 	my $self = shift;
 	my $err  = '';
-
+	
 	unless (@{$self->{'axfr_rr'}}) {
 		unless ($self->{'axfr_sel'}) {
-			$err = 'no zone transfer in progress';
+			my $err = 'no zone transfer in progress';
+			
+			print ";; $err\n" if $self->{'debug'};
 			$self->errorstring($err);
+					
 			return wantarray ? (undef, $err) : undef;
 		}
 
@@ -1160,6 +1033,10 @@ sub axfr_next {
 
 		if ($self->{'axfr_soa_count'} >= 2) {
 			$self->{'axfr_sel'} = undef;
+			# we need to mark the transfer as over if the responce was in 
+			# many answers.  Otherwise, the user will call axfr_next again
+			# and that will cause a 'no transfer in progress' error.
+			push(@{$self->{'axfr_rr'}}, undef);
 		}
 	}
 
@@ -1256,6 +1133,37 @@ sub AUTOLOAD {
 	goto &{$AUTOLOAD};	
 }
 
+__END__
 
+=head1 NAME
+
+Net::DNS::Resolver::Base - Common Resolver Class
+
+=head1 SYNOPSIS
+
+ use base qw/Net::DNS::Resolver::Base/;
+
+=head1 DESCRIPTION
+
+This class is the common base class for the different platform
+sub-classes of L<Net::DNS::Resolver|Net::DNS::Resolver>.  
+
+No user serviceable parts inside, see L<Net::DNS::Resolver|Net::DNS::Resolver>
+for all your resolving needs.
+
+=head1 COPYRIGHT
+
+Copyright (c) 1997-2002 Michael Fuhr. 
+
+Portions Copyright (c) 2002-2003 Chris Reinhardt.
+
+All rights reserved.  This program is free software; you may redistribute
+it and/or modify it under the same terms as Perl itself.
+
+=head1 SEE ALSO
+
+L<perl(1)>, L<Net::DNS>, L<Net::DNS::Resolver>
+
+=cut
 
 1;
