@@ -1,6 +1,6 @@
 package Net::DNS::Resolver::Win32;
 #
-# $Id: Win32.pm,v 1.3 2003/08/26 23:58:10 ctriv Exp $
+# $Id: Win32.pm,v 1.6 2003/09/24 22:38:51 ctriv Exp $
 #
 
 use strict;
@@ -9,7 +9,7 @@ use vars qw(@ISA $VERSION);
 use Net::DNS::Resolver::Base ();
 
 @ISA     = qw(Net::DNS::Resolver::Base);
-$VERSION = (qw$Revision: 1.3 $)[1];
+$VERSION = (qw$Revision: 1.6 $)[1];
 
 use Win32::Registry;
 
@@ -34,7 +34,7 @@ sub init {
 	# Best effort to find a useful domain name for the current host
 	# if domain ends up blank, we're probably (?) not connected anywhere
 	# a DNS server is interesting either...
-	my $domain      = $keys{'Domain'}->[2] || $keys{'DhcpDomain'}->[2];
+	my $domain = $keys{'Domain'}->[2] || $keys{'DhcpDomain'}->[2] || '';
 	
 	# If nothing else, the searchlist should probably contain our own domain
 	# also see below for domain name devolution if so configured
@@ -136,3 +136,34 @@ sub init {
 
 1;
 __END__
+
+
+=head1 NAME
+
+Net::DNS::Resolver::Win32 - Windows Resolver Class
+
+=head1 SYNOPSIS
+
+ use Net::DNS::Resolver;
+
+=head1 DESCRIPTION
+
+This class implements the windows specific portions of C<Net::DNS::Resolver>.
+
+No user serviceable parts inside, see L<Net::DNS::Resolver|Net::DNS::Resolver>
+for all your resolving needs.
+
+=head1 COPYRIGHT
+
+Copyright (c) 1997-2002 Michael Fuhr. 
+
+Portions Copyright (c) 2002-2003 Chris Reinhardt.
+
+All rights reserved.  This program is free software; you may redistribute
+it and/or modify it under the same terms as Perl itself.
+
+=head1 SEE ALSO
+
+L<perl(1)>, L<Net::DNS>, L<Net::DNS::Resolver>
+
+=cut
